from yarl import URL

from ..base_functions.base_functions import log, logger, make_title_safe
from ..base_functions.data_classes import DomainItem
from ..client.client import Session


class EromeCrawler():
    def __init__(self, *, include_id=False):
        self.include_id = include_id

    async def fetch(self, session: Session, url: URL):
        domain_obj = DomainItem(url.host, {})

        await log("Starting scrape of " + str(url))

        try:
            soup = await session.get_BS4(url)
            title = soup.select_one('div[class="col-sm-12 page-content"] h1').get_text()
            if title is None:
                title = url.name
            elif self.include_id:
                title = title + " - " + url.name
            title = await make_title_safe(title)

            # Images
            for link in soup.select('img[class="img-front lasyload"]'):
                await domain_obj.add_to_album(title, URL(link['data-src']), url)

            # Videos
            for link in soup.select('div[class=media-group] div[class=video-lg] video source'):
                await domain_obj.add_to_album(title, URL(link['src']), url)

        except Exception as e:
            logger.debug("Error encountered while handling %s", str(url), exc_info=True)
            await log("Error scraping " + str(url))
            logger.debug(e)

        await log("Finished scrape of " + str(url))

        return domain_obj



config_default = [
    {
        "Configuration": {
            "Apply_Config": False,
            "Files": {
                "input_file": "URLs.txt",
                "output_folder": "Downloads",
                "output_last_forum_post_file": "URLs_last_post.txt",
                "db_file": "download_history.sqlite",
            },
            "Authentication": {
                "simpcity_username": "",
                "simpcity_password": "",
                "socialmediagirls_username": "",
                "socialmediagirls_password": "",
                "xbunker_username": "",
                "xbunker_password": "",
            },
            "JDownloader": {
                "jdownloader_enable": False,
                "jdownloader_username": "",
                "jdownloader_password": "",
                "jdownloader_device": "",
            },
            "Runtime": {
                "threads": 0,
                "attempts": 10,
                "disable_attempt_limit": False,
                "connection_timeout": 15,
                "ratelimit": 50,
                "throttle": 0.5,
                "include_id": False,
                "exclude_videos": False,
                "exclude_images": False,
                "exclude_audio": False,
                "exclude_other": False,
                "ignore_history": False,
                "output_last_forum_post": False,
                "separate_posts": False,
                "mark_downloaded": False,
                "proxy": "",
                "skip_hosts": [],
            }
        }
    }
]

files_args = ["input_file", "output_folder", "output_last_forum_post_file", "db_file"]
authentication_args = ["simpcity_username", "simpcity_password",
                       "socialmediagirls_username", "socialmediagirls_password",
                       "xbunker_username", "xbunker_password"]
jdownloader_args = ["jdownloader_enable", "jdownloader_username", "jdownloader_password", "jdownloader_device"]
runtime_args = ["threads", "attempts", "disable_attempt_limit", "connection_timeout", "ratelimit", "throttle",
                "include_id", "exclude_videos", "exclude_images", "exclude_audio", "exclude_other", "ignore_history",
                "output_last_forum_post", "mark_downloaded", "separate_posts", "skip_hosts", "proxy"]



from yarl import URL

from cyberdrop_dl.base_functions.base_functions import log, logger
from ..client.client import Session


class RedGifsCrawler:
    async def fetch(self, session: Session, url: URL):
        try:
            soup = await session.get_BS4(url)
            video_metas = soup.select("meta[property='og:video']")

            # Find the video that doesn't contain "mobile"
            video_src = None
            for video_meta in video_metas:
                link = video_meta.get("content")
                if "mobile" not in link:
                    video_src = URL(link)

            # Just get the first one if we didn't find the non-mobile
            if video_src is None and len(video_metas) != 0:
                video_src = URL(video_metas[0].get("content"))

            return video_src

        except Exception as e:
            logger.debug("Error encountered while handling %s", str(url), exc_info=True)
            await log("Error scraping " + str(url))
            logger.debug(e)



from xml.etree.ElementTree import Element
from yarl import URL

from ..base_functions.base_functions import log, logger
from ..base_functions.data_classes import DomainItem
from ..client.client import Session


class Rule34Crawler:
    async def fetch(self, session: Session, url: URL):
        try:
            # get the &id=1234 part from the url
            # an url can look like this:
            # https://rule34.xxx/index.php?page=post&s=view&id=6766683
            id = url.query['id']

            # call the api for
            # https://api.rule34.xxx/index.php?page=dapi&s=post&q=index&id=${OUR_ID}
            apiUrl = URL("https://api.rule34.xxx/index.php?page=dapi&s=post&q=index&id=" + id)
            response = await session.get_xml(apiUrl)

            for child in response:
                if child.tag != "post":
                    continue
                file_url = child.attrib["file_url"]
                break

            domain_object = DomainItem("rule34.xxx", {})
            await domain_object.add_to_album("Rule34", URL(file_url), url)
            
            return domain_object

        except Exception as e:
            logger.debug("Error encountered while handling %s", str(url), exc_info=True)
            await log("Error scraping " + str(url))
            logger.debug(e)



from yarl import URL

from ..base_functions.base_functions import log, logger
from ..client.client import Session


class GfycatCrawler:
    async def fetch(self, session: Session, url: URL):
        try:
            soup = await session.get_BS4(url)

            video = soup.select_one("video[class='video media']")
            video_srcs = video.select("source")

            # Find the biggest video source, usually the host is giant.gyfcat
            video_link = None
            for src in video_srcs:
                link = URL(src.get("src"))
                if str(link.host).find("giant") != -1:
                    video_link = link
            if video_link is None:
                video_link = URL(video_srcs[0].get("src"))

            return video_link

        except Exception as e:
            logger.debug("Error encountered while handling %s", str(url), exc_info=True)
            await log("Error scraping " + str(url))
            logger.debug(e)
